package digital.laboratory.platform.imr.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.imr.dto.DrugHandingOverReviewDTO;
import digital.laboratory.platform.imr.entity.DrugHandingOverApply;
import digital.laboratory.platform.imr.query.DrugHandingOverApplyQuery;
import digital.laboratory.platform.imr.vo.DrugHandingOverApplyVO;

import java.util.List;

/**
* @author ChenJiangBao
* @description 针对表【b_drug_handing_over_apply(毒品库中关联的案事件信息)】的数据库操作Service
* @createDate 2025-01-08 17:30:46
*/
public interface DrugHandingOverApplyService extends IService<DrugHandingOverApply> {

    /**
     * 创建毒品送缴申请-委托单位操作
     * @return
     */
    DrugHandingOverApplyVO create();


    /**
     * 根据毒品送缴申请查询条件分页查询毒品送缴申请VO对象
     *
     * @param query 查询条件
     * @return 分页后的毒品送缴申请视图对象列表
     */
    IPage<DrugHandingOverApplyVO> voPage(DrugHandingOverApplyQuery query);

    /**
     * 删除
     * @param ids
     * @return
     */
    boolean delete(List<String> ids);

    /**
     * 审核送缴申请
     *
     * @param dto
     * @param user
     * @return
     */
    DrugHandingOverApplyVO review(DrugHandingOverReviewDTO dto, DLPUser user);

    /**
     * 提交送缴申请
     *
     * @param applyIds
     * @return
     */
    boolean submit(List<String> applyIds);

    /**
     * 完成指定的申请
     *
     * @param applyId 申请的唯一标识符
     * @return 如果申请成功完成，则返回true；否则返回false
     */
    boolean finish(String applyId);

    /**
     * 判断申请中是否有待入库或者出库待归还的检材
     * @param applyId
     * @return
     */
    boolean whetherAllowInbound(String applyId);

}
